"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticGenerator = void 0;
const path_1 = __importDefault(require("path"));
const ts = __importStar(require("typescript"));
const base_1 = require("@plutolang/base");
const utils_1 = require("./utils");
// The name of the compiled entrypoint
const ENTRYPOINT_FILENAME = "pulumi";
class StaticGenerator extends base_1.core.Generator {
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    name = require(path_1.default.join(__dirname, "../package.json")).name;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    version = require(path_1.default.join(__dirname, "../package.json")).version;
    constructor(args) {
        super(args);
    }
    async generate(archRef, outdir) {
        const pirTsCode = this.generateInfraCode(archRef);
        (0, utils_1.writeToFile)(outdir, ENTRYPOINT_FILENAME + ".ts", pirTsCode);
        const pirJsCode = compileTs(pirTsCode);
        (0, utils_1.writeToFile)(outdir, ENTRYPOINT_FILENAME + ".js", pirJsCode);
        return { entrypoint: path_1.default.join(outdir, ENTRYPOINT_FILENAME + ".js") };
    }
    generateInfraCode(archRef) {
        const entities = archRef.topoSort();
        const globalImports = `import { createClosure } from "@plutolang/base/closure";`;
        let infraCode = ``;
        for (const entity of entities) {
            if (entity instanceof base_1.arch.Resource) {
                infraCode += this.generateInfraCode_Resource(entity);
            }
            else if (entity instanceof base_1.arch.Closure) {
                infraCode += this.generateInfraCode_Closure(entity, archRef);
            }
            else if (entity instanceof base_1.arch.Relationship) {
                infraCode += this.generateInfraCode_Relationship(entity);
            }
        }
        // Append the postProcess calling for each resource.
        entities
            .filter((entity) => entity instanceof base_1.arch.Resource)
            .forEach((entity) => {
            const resource = entity;
            infraCode += `${resource.id}.postProcess();\n`;
        });
        // Append the output items of each resource.
        // TODO: Currently, these outputs are only utilized during testing. We need to evaluate their
        // necessity, as this approach requires the SDK developer to specifically write outputs for
        // certain resources, which may not be developer-friendly.
        const outputItems = entities
            .filter((entity) => entity instanceof base_1.arch.Resource)
            .map((entity) => {
            const resource = entity;
            return `${resource.id}: ${resource.id}.outputs`;
        });
        infraCode += `return {
${outputItems.join(",\n")}
}`;
        return `
${globalImports}

export default (async () => {
${infraCode}
})();
`;
    }
    generateInfraCode_Resource(resource) {
        const dotPos = resource.type.lastIndexOf(".");
        const pkgName = dotPos == -1 ? "@plutolang/pluto" : resource.type.substring(0, dotPos);
        const typeName = dotPos == -1 ? resource.type : resource.type.substring(dotPos + 1);
        return `
const ${resource.id} = await (
  await import("${pkgName}-infra")
).${typeName}.createInstance(${resource.getParamString()});
`;
    }
    generateInfraCode_Closure(closure, archRef) {
        // This section identifies all dependencies of the closure, which fall into two categories:
        // 1. Resources whose properties the closure accesses. For these resources, we need to transfer
        //    the properties to the runtime environment via environment variables.
        // 2. Resources whose methods the closure calls. For these resources, we need to request
        //    permissions so that the closure can invoke these methods during runtime on the platform.
        const dependencies = [];
        archRef.relationships
            .filter((relat) => relat.from.type === "closure" &&
            relat.from.id === closure.id &&
            relat.type !== base_1.arch.RelatType.Create)
            .forEach((relat) => {
            relat.to
                .filter((to) => to.type === "resource")
                .forEach((to) => {
                dependencies.push({
                    resourceId: to.id,
                    type: relat.type === base_1.arch.RelatType.MethodCall ? "method" : "property",
                    operation: relat.operation,
                });
            });
        });
        // Construct the dependency items and concatenate them using a comma separator.
        const dependenciesString = dependencies
            .map((dep) => `
{ 
  resourceObject: ${dep.resourceId}, 
  type: "${dep.type}", 
  operation: "${dep.operation}" 
}
`)
            .join(",");
        let exportName;
        switch (this.language) {
            case base_1.LanguageType.TypeScript:
                exportName = "default";
                break;
            case base_1.LanguageType.Python:
                exportName = "_default";
                break;
            default:
                throw new Error(`Unsupported language: ${this.language}`);
        }
        const dirpath = path_1.default.resolve(this.rootpath, closure.path);
        // We encapsulate the closure within a function because the statements in the closure's global
        // scope are executed upon import. However, these statements are likely intended to run on the
        // target platform, not during the deployment stage.
        return `
const ${closure.id}_func = async (...args: any[]) => {
  const handler = (await import("${dirpath}")).default;
  return await handler(...args);
}
const ${closure.id} = createClosure(${closure.id}_func, {
  dirpath: "${dirpath}",
  exportName: "${exportName}",
  dependencies: [${dependenciesString}],
});
`;
    }
    generateInfraCode_Relationship(relationship) {
        return `
${relationship.from.id}.${relationship.operation}(${relationship.getParamString()});
`;
    }
}
exports.StaticGenerator = StaticGenerator;
function compileTs(code) {
    const result = ts.transpileModule(code, {
        compilerOptions: { module: ts.ModuleKind.CommonJS },
    });
    if (result.diagnostics) {
        result.diagnostics.forEach((diagnostic) => {
            if (diagnostic.file) {
                const { line, character } = ts.getLineAndCharacterOfPosition(diagnostic.file, diagnostic.start);
                const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n");
                console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
            }
            else {
                console.log(ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n"));
            }
        });
    }
    return result.outputText;
}
//# sourceMappingURL=generator.js.map